******************************************************
*  Program...........: L7UtilSamples.prg
*  Author............: RandyPearson
*  Project...........: L7
*  Copyright.........: Cycla Corp

******************************************************
* COMMENTS
* This file contains the samples from the L7Utils.prg 
*

* --------------------------------------------------------- *
FUNCTION L7RegexpAT( lcPattern, lcSourceString, lnOcc, llIgCase, llRTOL )
  LOCAL loObj, lnResult
  loObj = CREATEOBJECT("L7RegExp")
  lnResult = loObj.AT(m.lcPattern, @lcSourceString, m.lnOcc, m.llIgCase, m.llRTOL)
  loObj = NULL
  RETURN m.lnResult
ENDFUNC  && L7RegExpAT
* --------------------------------------------------------- *
FUNCTION L7RegexpRAT(lcPattern AS String, lcSourceString AS String, lnOcc AS Integer)
  RETURN L7RegexpAT(m.lcPattern, @lcSourceString, m.lnOcc, .F., .T.)
ENDFUNC
* --------------------------------------------------------- *
FUNCTION L7RegexpRATC(lcPattern AS String, lcSourceString AS String, lnOcc AS Integer)
  RETURN L7RegexpAT(m.lcPattern, @lcSourceString, m.lnOcc, .T., .T.)
ENDFUNC
* --------------------------------------------------------- *
FUNCTION L7RegexpATC(lcPattern AS String, lcSourceString AS String, lnOcc AS Integer)
  RETURN L7RegexpAT(m.lcPattern, @lcSourceString, m.lnOcc, .T.)
ENDFUNC

* --------------------------------------------------------- *
FUNCTION L7RegexpOccurs( tcText, tcPattern ) as Integer
  * Like L7RegExpTest, but return number of matches.
  * Examples:
  *   ? L7RegexpOccurs('abracadabra', 'ab')     && 2
  *   ? L7RegexpOccurs('abracadabra', '(ab\w+)')  && 1 (greedy)
  *   ? L7RegexpOccurs('abracadabra', '(ab\w+)?') && 2 (non-greedy)
  *
  LOCAL loObj, loMatches, lnResult
  loObj = CREATEOBJECT("L7RegExp")
  loObj.Pattern = m.tcPattern
  loMatches = loObj.Execute( m.tcText )
  lnResult = loMatches.Count
  loMatches = NULL
  loObj = NULL
  RETURN m.lnResult
ENDFUNC  && L7RegexpOccurs

* --------------------------------------------------------- *
FUNCTION L7RegexpTest( tcText, tcPattern ) as Boolean
  * same as perl's m
  * Examples:
  *   ? L7RegexpTest('abracadabra', '(ab)')     && .T.
  *
  LOCAL loObj, llResult
  loObj = CREATEOBJECT("L7RegExp")
  loObj.Pattern = m.tcPattern
  llResult = loObj.Test( m.tcText )
  loObj = NULL
  RETURN m.llResult
ENDFUNC  && L7RegexpTest

* --------------------------------------------------------- *
FUNCTION L7RegexpReplace( tcText, tcSearchPattern, tcReplacePattern ) as String
  * mimics perl's 's' function -- EXCEPT
  * returns STRING instead of hit count -- 
  * can also pass tcText by reference to get replacement that way
  * Examples:
  *   ? L7RegexpReplace('abracadabra', 'ab', 'cd')     && "cdracadcdra"
  *
  * or with pass by reference:
  *
  *   lcText = 'abracadabra'
  *   L7RegexpReplace(@lcText, 'ab', 'cd')
  *
  LOCAL loObj, lnResult, loMatches
  loObj = CREATEOBJECT("L7RegExp")
  loObj.Pattern = m.tcSearchPattern
  *!* Wasted step, just to get hit count:
  *!* loMatches = loObj.Execute( m.tcText )
  *!* lnResult = loMatches.Count
  * Now do the real thing:
  *!* IF m.lnResult > 0
  tcText = loObj.Replace(m.tcText, m.tcReplacePattern)
  *!* ENDIF
  loObj = NULL
  *!* RETURN m.lnResult
  RETURN m.tcText
ENDFUNC  && L7RegexpReplace

*** ========================================================= ***
DEFINE CLASS L7RegExp AS LINE
  cRegexpClass = "VBScript.RegExp"
  Pattern      = [*]
  MultiLine    = .F.
  Global       = .T.
  IgnoreCase   = .T.
  * --------------------------------------------------------- *
  FUNCTION AT(lcPattern AS String, lcSourceString AS String, lnOcc AS Integer, llIgnoreCase AS Boolean, llFromRight AS Boolean)
    THIS.IgnoreCase = m.llIgnoreCase
    THIS.MultiLine  = .T.
    THIS.Global     = .T.
    THIS.Pattern    = m.lcPattern
    lnOcc = EVL(m.lnOcc, 1)
    LOCAL loObj, lnResult, loMatches, loMatch, lnCount
    lnResult = 0
    loObj = CREATEOBJECT(THIS.cRegexpClass)
    THIS.ConfigRegExp(m.loObj)
    loMatches = loObj.Execute(m.lcSourceString)
    lnCount = m.loMatches.Count
    IF m.lnCount >= m.lnOcc
      loMatch = loMatches.item[IIF(m.llFromRight, m.lnCount - m.lnOcc, m.lnOcc - 1)]
      lnResult = loMatch.FirstIndex + 1
    ENDIF
    RETURN m.lnResult
  ENDFUNC  && AT()
  * --------------------------------------------------------- *
  FUNCTION RAT(lcPattern AS String, lcSourceString AS String, lnOcc AS Integer)
    RETURN THIS.AT(m.lcPattern, @lcSourceString, m.lnOcc, .F., .T.)
  ENDFUNC
  * --------------------------------------------------------- *
  FUNCTION RATC(lcPattern AS String, lcSourceString AS String, lnOcc AS Integer)
    RETURN THIS.AT(m.lcPattern, @lcSourceString, m.lnOcc, .T., .T.)
  ENDFUNC
  * --------------------------------------------------------- *
  FUNCTION ATC(lcPattern AS String, lcSourceString AS String, lnOcc AS Integer)
    RETURN THIS.AT(m.lcPattern, @lcSourceString, m.lnOcc, .T.)
  ENDFUNC
  * --------------------------------------------------------- *
  FUNCTION Test(lcSourceString AS String) AS Boolean 
    LOCAL loObj, llResult
    * Remove and create new object on every request, because
    * we've noted some corruption in the COM object:
    loObj = CREATEOBJECT(THIS.cRegexpClass)
    THIS.ConfigRegExp(m.loObj)
    TRY
      llResult = loObj.Test(m.lcSourceString)
    CATCH TO loExc WHEN loExc.ErrorNo = 1426
      THROW "COM error testing regular expression."
    ENDTRY
    loObj = NULL
    RETURN m.llResult
  ENDFUNC  && Test()
  * --------------------------------------------------------- *
  FUNCTION Replace(lcSourceString AS String, lcReplaceString AS String) AS String
    LOCAL loObj, lcResult
    * Remove and create new object on every request, because
    * we've noted some corruption in the COM object:
    loObj = CREATEOBJECT(THIS.cRegexpClass)
    THIS.ConfigRegExp(m.loObj)
    lcResult = loObj.Replace(m.lcSourceString, m.lcReplaceString)
    loObj = NULL
    RETURN m.lcResult
  ENDFUNC  && Replace()
  * --------------------------------------------------------- *
  FUNCTION Execute(lcSourceString AS String) AS Object
    LOCAL loObj, lnNewMatch, loMatches, loMatch, loRet AS Collection, lcSub 
    * Remove and create new object on every request, because
    * we've noted some corruption in the COM object:
    loObj = CREATEOBJECT(THIS.cRegexpClass)
    THIS.ConfigRegExp(m.loObj)
    loMatches = loObj.Execute(m.lcSourceString)
    loRet = CREATEOBJECT("Collection")
    FOR EACH loMatch IN loMatches
      loNewMatch = CREATEOBJECT("L7RegExpMatch")
      WITH loNewMatch
        .Value = loMatch.Value
        .Length = loMatch.Length
        FOR EACH lcSub IN loMatch.SubMatches
          .SubMatches.Add(m.lcSub)
        ENDFOR
      ENDWITH
      loRet.Add(loNewMatch)
    ENDFOR
    loObj = NULL
    RETURN m.loRet
  ENDFUNC  && Execute()
  * --------------------------------------------------------- *
  FUNCTION ConfigRegExp(loObj)
    WITH loObj
      .Pattern    = THIS.Pattern
      .Global     = THIS.Global
      .IgnoreCase = THIS.IgnoreCase
      .MultiLine  = THIS.MultiLine
    ENDWITH
  ENDFUNC  
  * --------------------------------------------------------- *
ENDDEFINE  && L7RegExp

*** ========================================================= ***
DEFINE CLASS L7RegExpMatch AS Container
  Value = NULL
  Length = 0
  ADD OBJECT SubMatches AS Collection
ENDDEFINE  && L7RegExpMatch

*** ========================================================== ***
DEFINE CLASS L7Lightweight AS RELATION

  HIDDEN CHILDALIAS
  HIDDEN CHILDORDER
  HIDDEN ONETOMANY
  HIDDEN PARENTALIAS
  HIDDEN RELATIONALEXPR
  
ENDDEFINE


* ------------------------------------------------------------------- *
FUNCTION AllTrimX( lcString)
  * Take out unwanted characters at beginning and end of string.
  * Can also pass by reference.
  IF VARTYPE( m.lcString) <> "C"
    lcString = ""
  ELSE
    LOCAL lcBad
    lcBad = CHR(10) + CHR(13) + CHR(9)
    DO WHILE .T.
    	* First a normal trim. This is done inside the loop,
      * so when we remove a CR, we then also remove further
      * embedded spaces, etc:
    	lcString = ALLTRIM( m.lcString)
    	DO CASE
    	CASE EMPTY( m.lcString)
    		EXIT
    	CASE RIGHT( m.lcString, 1) $ m.lcBad
    		lcString = LEFT( m.lcString, LEN( m.lcString) - 1)
    	CASE LEFT( m.lcString, 1) $ m.lcBad
    		lcString = SUBSTR( m.lcString, 2)	
    	OTHERWISE
    		EXIT
    	ENDCASE
    ENDDO
  ENDIF
  RETURN m.lcString
ENDFUNC  && AllTrimX