*** This security manager provides security by exception
*** This is done so that, if a form or a menu option is available
*** and editable by all users, no record needs to be added to sys_menu
*** or sys_form. The only fields that should go into sys_field are fields
*** that should not inherit the form's mode. For example, if a user has
*** only view rights (as opposed to edit rights) on a given form, 
*** all controls automatically inherit the form's access mode. Typically,
*** the only fields in sys_field will be fields that are "sensitive" so that
*** they can be included in sys_user_field with an access level of 'I' meaning
*** that they will not be visible for that user
DEFINE CLASS SecurityMgr AS Session 
  user_id = -1
  formDS = 1
  
******************************************************************************
FUNCTION INIT()
******************************************************************************
LOCAL llRetVal
  llRetVal = DODEFAULT()
  IF llRetVal
    *** Set the default environment
    This.SetEnvironment()
  ENDIF  && llRetVal
  RETURN llRetVal
ENDFUNC

******************************************************************************
FUNCTION user_id_assign( tiUserID )
******************************************************************************
  WITH This
    IF VARTYPE( tiUserID ) = "N" AND NOT EMPTY( tiUserID )
      *** We have a potentially valid user id
      .user_id = tuValue
      .GetSecurityData()
    ENDIF
    RETURN
  ENDWITH
ENDFUNC

*****************************************************************************
FUNCTION ChkModule( tcModuleName )
*****************************************************************************
*** All modules that are not available to all users must be in the sys_module table
*** Security is by exception - if this user does not have access to this module
*** you will find an entry in sys_user_module.
*** To see if a user has access to a given module, call the security object like this:
*** llRetVal = oSecurityMgr.ChkModule( <Module Name> )
*** a return value of .F. means that the current user does not have access
LOCAL llRetVal
  *** See if this module is restricted for this user
  SELECT lv_restricted_module
  LOCATE FOR ALLTRIM( module_nme ) == LOWER( ALLTRIM( tcModuleName ) )

  llRetVal = NOT( FOUND() )
  RETURN llRetVal
ENDFUNC

*****************************************************************************
FUNCTION ChkMenu( tcMenuKey )
*****************************************************************************
*** This function assumes that GenMenuX was used as the menu builder
*** to generate the .mpr. To call this function, just put something
*** that lloks like this:
***     *:IF menuchk("ACCOUNTING") 
*** in the comment field of the menu designer. GenMenuX generates
*** all the code required to remove menu pads that the user does
*** not have access to. The passed parameter must be the same as
*** the sys_menu_key field in the sys_menu table. All menu pad and
*** bar names that are not automatically available to all users 
*** must be in the sys_menu table
LOCAL llRetVal

*** See if this menu is restricted for this user
*** for this menu item
SELECT lv_restricted_menu
LOCATE FOR ALLTRIM( menu_key ) == LOWER( ALLTRIM( tcMenuKey ) )
llRetVal = NOT( FOUND() )

RETURN llRetVal
  
ENDFUNC

***********************************************************************
FUNCTION GetSecurityData()
********************************************************************
LOCAL vp_user_id

  *** get all the restrictions for this logged in user
  vp_user_id = This.user_id

  *** first get restricted modules
  USE lv_restricted_module IN 0

  *** next get restructed menus
  USE lv_restricted_menu IN 0

  *** also get form access restricions
  USE lv_restricted_form IN 0

  *** and sensitive field that are restricted
  USE lv_restricted_field IN 0

ENDFUNC

********************************************************************
*** [E] SETENVIRONMENT: Set the default environment
********************************************************************
FUNCTION SetEnvironment()
  WITH This
    *** If we are in Dev Mode, set Escape and Asserts
    IF VERSION( 2 ) # 0
      SET ESCAPE ON
      SET ASSERTS ON
    ELSE
      SET ESCAPE OFF
      SET ASSERTS OFF
      SET STATUS BAR OFF
    ENDIF
    SET SYSFORMATS OFF
    SET CENTURY ON ROLLOVER 60
    SET ANSI OFF
    SET CONFIRM ON
    SET DELETED ON
    SET EXACT OFF
    SET EXCLUSIVE OFF
    SET MULTILOCKS ON
    *** Is the first day of the week Monday? For a business, I would think so.
    SET FDOW TO 2
    SET FULLPATH ON
    SET MEMOWIDTH TO 50
    SET NOTIFY OFF
    SET NULL OFF
    SET NULLDISPLAY TO ' '
    SET REPROCESS TO 10 SECONDS
    SET SAFETY OFF
    SET STATUS OFF
    SET STRICTDATE TO 0
    SET TEXTMERGE ON
  ENDWITH

ENDDEFINE